==> Mudlib-CD.00.28.tar.gz: lib/std/player_sec.c <==
/*

  /std/player_sec.c

  This is the security part of the old player.c.

  This file is statically inherited by player_pub to ensure the
  protection of all lower level routines.

*/

#pragma save_binary

inherit "/std/living";

#include "/std/player/savevars_sec.c"	/* Variables saved to player file */
#include "/std/player/cmd_sec.c"
#include "/std/player/getmsg_sec.c"
#include "/std/player/death_sec.c"
#include "/std/player/querys_sec.c"
#include "/std/player/pcombat.c"
#include "/std/player/quicktyper.c"

#include <macros.h>
#include <language.h>
#include <ss_types.h>
#include <money.h>
#include <std.h>
#include <stdproperties.h>

/*
 * Variables used by the quicktyper
 */
private static string last_verb, last_str;
private static int    history_num;

#ifdef RELAY_HOST
static string real_host;

public nomask string query_real_host() { return real_host; }
#endif

/*
 * Prototypes
 */
void new_init();
void load_auto_obj(string *load_arr);
int load_recover_list(string pl_name, string *recover_arr);
void load_auto_shadow(string *load_arr);
static void init_saved_props();

/*
 * Function name: query_def_start
 * Description:   Return the default starting location of the player type
 *                This function is supposed to be replaced in inheriting
 *                player objects.
 */
public string
query_def_start() { return DEFAULT_START; }

/*
 * Function name: query_orig_stat
 * Description:   Return the default starting stats of a player
 *                This function is supposed to be replaced in inheriting
 *                player objects.
 */
public int *
query_orig_stat() 
{
    int i, *list;

    list = ({});
    for (i = 0; i < SS_NO_STATS; i++)
	list += ({ 1 });
    return list;
}

/*
 * Function name: query_orig_learn
 * Description:   Return the default starting stats of a player
 *                This function is supposed to be replaced in inheriting
 *                player objects.
 */
public int *
query_orig_learn() 
{
    int i, *list;
    
    list = ({});
    for (i = 0; i < SS_NO_EXP_STATS; i++)
	list += ({ 100 / SS_NO_EXP_STATS });
    return list;
}

/*
 * Function name: query_new_title
 * Description:   Return the default starting title of a player
 *                This function is supposed to be replaced in inheriting
 *                player objects.
 */

public string
query_new_title() { return "the utter novice"; }

/*
 * Function name: query_new_al_title
 * Description:   Return the default starting title of a player
 *                This function is supposed to be replaced in inheriting
 *                player objects.
 */
public string
query_new_al_title() { return "neutral"; }

/*
 * Function name: enter_game
 * Description:   Enter the player into the game
 * Arguments:	  pl_name: Name of the player
 * Security:
 *           This function must only be called from "/secure/login"
 */
public nomask int
enter_game(string pl_name)
{
    string      fname;
    string	*souls;
    int 	il;
    int		lost_money;
    object 	ob;

    if (MASTER_OB(previous_object()) != "/secure/login" &&
	MASTER_OB(previous_object()) != LOGIN_NEW_PLAYER)
    {
	write("Bad login object: " + file_name(previous_object()) + "\n");
	return 0;
    }
#ifdef RELAY_HOST
    else
	real_host = previous_object()->query_relay();
#endif

    if (SECURITY->check_inherit("/std/player_sec", this_object()))
    {
	remove_object();
	return 0;
    }
    set_name(pl_name);

    ::set_adj(({}));		/* No adjectives and no default */
    new_init();             	/* All variables to default condition */

    seteuid(0);
    if (!SECURITY->load_player())
    {
	return 0;
    }
    seteuid(query_wiz_level() ? pl_name : getuid(this_object()));
    set_adj(0);			/* Set the adjectives as loaded */

    add_name(query_race_name());
    if (query_wiz_level())
	add_name("wizard");

    /* Make some sanity things to guard against old and patched .o files
    */
    set_m_out(query_m_out());
    set_m_in(query_m_in());
    set_mm_out(query_mm_out());
    set_mm_in(query_mm_in());
    set_learn_pref(query_learn_pref(-1));

    /* People might have inconsistent acc_exp in guildstats, we must guard
       against it.
    */
    il = this_object()->query_exp();
    this_object()->add_exp(1, 1);
    if (this_object()->query_exp() < il)
    {
	il -= this_object()->query_exp();
	this_object()->add_exp(il);
    }
    else
	this_object()->add_exp(-1, 1);

    set_login_time();
    set_login_from();

    set_living_name(pl_name);
    cmd_sec_reset();
    
    acc_exp_to_stats();   /* Setup our current stats */

    if (query_temp_start_location() &&
	SECURITY->check_temp_start_loc(query_temp_start_location()) >= 0)
    {
	catch(this_object()->move_living(0, query_temp_start_location()));
	set_temp_start_location(0);
    }

    if (!environment(this_object()))
    {
	if (!query_default_start_location() ||
	    (!query_wiz_level() && SECURITY->
		check_def_start_loc(query_default_start_location()) < 0))
	    set_default_start_location((string) this_object()->
		query_def_start());
	catch(this_object()->move_living(0, query_default_start_location()));
    }

    /*
     * Let players start even if their start location is bad
     */
    if (!environment(this_object()) && !query_wiz_level())
	if (catch(this_object()->move_living(0, query_def_start())))
	{
	    /*
	     * If this start location is corrupt too, then destruct the player
	     */
	    write("PANIC, your starting locations are corrupt!!\n");
	    destruct(this_object());
    	}

    /* Set some default properties */
    this_object()->add_prop(PLAYER_I_MORE_LEN, 20);
    /* Restore the bits */
    this_object()->unpack_bits();

    if ((query_wiz_level()) && (environment(this_object())))
	write("Starting in:" + file_name(environment(this_object())) + "\n");

    init_saved_props();

    /* Non wizards should not have a lot of command souls
     */
    if (!query_wiz_level())
    {
	souls = this_object()->query_cmdsoul_list();
	if (sizeof(souls))
	{
	    for (il = 0; il < sizeof(souls); il++)
		remove_cmdsoul(souls[il]);
	}
    }

    /* Attach the quicktyper */
    if (!this_object()->query_cmdmodify())
    {
	this_object()->set_cmdmodify(this_object()); /* Built in QT */
    }

    if (!m_alias_list)
	m_alias_list = ([]);

    set_heart_beat(1);

    /* Start him up */
    if (query_autoshadow_list())
	load_auto_shadow(query_autoshadow_list());
    if (query_auto_load())
	load_auto_obj(query_auto_load());
    if (query_recover_list())
	if (load_recover_list(pl_name, query_recover_list()) == 0)
	    set_recover_list(({}));

    /* Start him up 
     */
    this_object()->start_player();
    this_object()->query_combat_object()->cb_configure();
    
    /* Non wizards should not have a lot of tool souls
     */
    if (!query_wiz_level())
    {
	souls = this_object()->query_tool_list();
	if (sizeof(souls))
	{
	    for (il = 0; il < sizeof(souls); il++)
		this_object()->remove_toolsoul(souls[il]);
	}
    }

    if ((lost_money = query_tot_value()) > 0)
    {
	tell_object(this_object(), "You find " + lost_money +
			" of your lost money.\n");
    	MONEY_ADD(this_object(), query_tot_value());
    	call_out("save_me", 1, 0);
    }
    add_prop(PLAYER_I_LASTXP, query_exp());

    return 1;
}

static void
init_saved_props()
{
    int i, n;

    n = sizeof(SAVE_PROPS) < sizeof(saved_props) ? sizeof(SAVE_PROPS) :
	sizeof(saved_props);

    for (i = 0; i < n; i++)
	this_object()->add_prop(SAVE_PROPS[i], saved_props[i]);

    saved_props = ({ });
}

public nomask void
open_player()
{
    if ((previous_object() == find_object(SECURITY)) ||
	(MASTER_OB(previous_object()) == "/secure/login"))
	seteuid(0);
}

nomask public int
fix_saveprop_list()
{
    int i;

    /* Fix the saved_props list before save */
    saved_props = ({ });
    for (i = 0; i < sizeof(SAVE_PROPS); i++)
	saved_props += ({ this_object()->query_prop(SAVE_PROPS[i]) });
}

nomask public int
save_player(string pl_name)
{
    int ret;
   
    if (!pl_name)
	return 0;

    this_object()->pack_bits();
    seteuid(getuid(this_object()));
    save_object(PLAYER_FILE(pl_name));
    seteuid(getuid(this_object()));

    /* Discard the props again */
    saved_props = ({ });
    return 1;
}

nomask public int
load_player(string pl_name)
{
    int ret;
   
    if (!pl_name)
	return 0;

    seteuid(getuid(this_object()));
    ret = restore_object(PLAYER_FILE(pl_name));
    seteuid(0);
    return ret;
}

/*
 * Function name: load_auto_obj
 * Description:   Loads all autoloaded objects
 */
nomask static void
load_auto_obj(string *load_arr)
{
    string          file,
                    argument,
                    *parts;
    object          ob;
    int		    il;

    if (!load_arr || !sizeof(load_arr))
	return;

    for (il = 0; il < sizeof(load_arr); il++)
    {
	if (sscanf(load_arr[il], "%s:%s", file, argument) != 2)
	{
	    file = load_arr[il]; argument = 0;
	}
	if (!stringp(file))
	{
	    write("Auto load string corrupt: " + load_arr[il] + "\n");
	    continue;
	}
	if (LOAD_ERR(file))
	    continue;
	ob = find_object(file);
	if (!ob)
	    continue;
	ob = clone_object(file);
	if (argument)
	    ob->init_arg(argument);
	ob->move(this_object());
    }
}

/*
 * Function name: load_recover_list
 * Description:   Loads all recoverable objects.
 * Arguments:	  pl_name - the player name,
 *		  recover_arr - The array of objects to recover
 */
nomask static int
load_recover_list(string pl_name, string *recover_arr)
{
    string          file,
                    argument,
                    *parts;
    object          ob;
    int		    il;

    if (!recover_arr || !sizeof(recover_arr))
	return 0;

    /* Do not restore after 2 hours */
    if (time() - file_time(PLAYER_FILE(pl_name) + ".o") > 3600 * 2)
	return 0;

    for (il = 0; il < sizeof(recover_arr); il++)
    {
	if (sscanf(recover_arr[il], "%s:%s", file, argument) != 2)
	{
	    file = recover_arr[il]; argument = 0;
	}
	if (!stringp(file))
	{
	    write("Recover string corrupt: " + recover_arr[il] + "\n");
	    continue;
	}
	if (LOAD_ERR(file))
	    continue;
	ob = find_object(file);
	if (!ob)
	    continue;
	ob = clone_object(file);
        write("You recover your " + ob->short() + ".\n");
	if (argument)
	    ob->init_recover(argument);
	ob->move(this_object());
    }
}

/*
 * Function name: load_auto_shadow
 * Description:   Startup all the shadows that should shadow this player
 */
nomask static void
load_auto_shadow(string *load_arr)
{
    string          file,
                    argument,
                    *parts;
    object          ob;
    int		    il;

    if (!load_arr || !sizeof(load_arr))
	return;

    for (il = 0; il < sizeof(load_arr); il++)
    {
	if (sscanf(load_arr[il], "%s:%s", file, argument) != 2)
	{
	    write("Shadow load string corrupt: " + load_arr[il] + "\n");
	    continue;
	}
	if (LOAD_ERR(file))
	    continue;
	ob = find_object(file);
	if (!ob)
	    continue;
	ob = clone_object(file);
	if (argument)
	    ob->autoload_shadow(argument);
	else
	    ob->autoload_shadow(0);
    }
}

/*
 * Function name: new_save
 * Description:   Save this player as 'name'
 *                This is only called when new players
 *                enters the game. It makes it possible to initialize variables
 *                using the standard set_ calls.
 * Arguments:	  pl_name: Name of the player
 * Security:
 *           This function must only be called from "/secure/login" or
 *           from the object creating new players. Which inherits
 */
public nomask int
new_save(string pl_name, string pwd, string pfile)
{
    string          fname;

    if (!CALL_BY("/secure/login") &&
	!CALL_BY(LOGIN_NEW_PLAYER))
	return 0;

    write("Creating new player: " + pl_name + "\n");
    seteuid(getuid(this_object()));
    set_name(pl_name);
    set_password(pwd);
    set_player_file(pfile);
    new_init();			/* Initialize all variables on startup */

    save_object(PLAYER_FILE(pl_name));
    return 1;
}


/*
 * Function:     new_init
 * Description:  Initialises all variables to default conditions.
 */
static nomask void
new_init()
{
    int i, *ostat;

    ostat = query_orig_stat();
    for (i = 0; i < SS_NO_STATS - 3; i++)
	set_base_stat(i, ostat[i]);

    stats_to_acc_exp();

    set_learn_pref(query_orig_learn());

    set_title(query_new_title());
#ifndef NO_ALIGN_TITLE
    set_al_title(query_new_al_title());
#endif
}

public nomask void
create_living()
{
    1;
}				/* We do not export any create call */

public nomask void
reset_living()
{
    1;
}				/* We do not export any reset call */

/*
 * Function name:	command
 * Description:		Makes the player execute a command, If the player is a
 *			wizard then the object must have the same effective
 *			userid as the wizard being forced.
 * Arguments:		cmd: String containing the command
 * Returns:		eval_cost or '0' if unsuccessful
 */
public nomask int
command(string cmd)
{
    object pobj;
    
    pobj = previous_object();
    if (query_wiz_level() && pobj && this_object())
        if (!SECURITY->wiz_force_check(geteuid(pobj), geteuid(this_object())))
	    return 0;

    return ::command(cmd);
}

nomask public int
follow_leader(string cmd, object leader)
{
    if (query_wiz_level())
    { /* Check for move commands */
	if (member_array(cmd, environment(this_object())->query_exit()) < 0)
	{
	    call_out("move_to_leader", 1);
	    return 0;
	}
    }
    return ::follow_leader(cmd, leader);
}

public nomask void
move_to_leader()
{
    if (query_leader())
	command("goto " + query_leader()->query_real_name());
}

public string
query_alias(string alias)
{
    mapping m;

    m = query_aliases();
    return(m[alias]);
}

public int
id(string str)
{
    if (notmet_me(previous_object()))
	if (str == query_real_name())
	    return 0;

    return ::id(str);
}

==> Mudlib-CD.00.30.tar.gz: lib/std/player_sec.c <==
/*

  /std/player_sec.c

  This is the security part of the old player.c.

  This file is statically inherited by player_pub to ensure the
  protection of all lower level routines.

*/

#pragma save_binary

inherit "/std/living";

#include "/std/player/savevars_sec.c"	/* Variables saved to player file */
#include "/std/player/cmd_sec.c"
#include "/std/player/getmsg_sec.c"
#include "/std/player/death_sec.c"
#include "/std/player/querys_sec.c"
#include "/std/player/pcombat.c"
#include "/std/player/quicktyper.c"

#include <macros.h>
#include <language.h>
#include <ss_types.h>
#include <money.h>
#include <std.h>
#include <stdproperties.h>

/*
 * Variables used by the quicktyper
 */
private static string last_verb, last_str;
private static int    history_num;

#ifdef RELAY_HOST
static string real_host;

public nomask string query_real_host() { return real_host; }
#endif

/*
 * Prototypes
 */
void new_init();
void load_auto_obj(string *load_arr);
int load_recover_list(string pl_name, string *recover_arr);
void load_auto_shadow(string *load_arr);
static void init_saved_props();

/*
 * Function name: query_def_start
 * Description:   Return the default starting location of the player type
 *                This function is supposed to be replaced in inheriting
 *                player objects.
 */
public string
query_def_start() { return DEFAULT_START; }

/*
 * Function name: query_orig_stat
 * Description:   Return the default starting stats of a player
 *                This function is supposed to be replaced in inheriting
 *                player objects.
 */
public int *
query_orig_stat() 
{
    int i, *list;

    list = ({});
    for (i = 0; i < SS_NO_STATS; i++)
	list += ({ 1 });
    return list;
}

/*
 * Function name: query_orig_learn
 * Description:   Return the default starting stats of a player
 *                This function is supposed to be replaced in inheriting
 *                player objects.
 */
public int *
query_orig_learn() 
{
    int i, *list;
    
    list = ({});
    for (i = 0; i < SS_NO_EXP_STATS; i++)
	list += ({ 100 / SS_NO_EXP_STATS });
    return list;
}

/*
 * Function name: query_new_title
 * Description:   Return the default starting title of a player
 *                This function is supposed to be replaced in inheriting
 *                player objects.
 */

public string
query_new_title() { return "the utter novice"; }

/*
 * Function name: query_new_al_title
 * Description:   Return the default starting title of a player
 *                This function is supposed to be replaced in inheriting
 *                player objects.
 */
public string
query_new_al_title() { return "neutral"; }

/*
 * Function name: fixup_screen
 * Description:   Restore the players screen width. Normally called
 *                during login.
 */
public nomask void
fixup_screen()
{
    int width;

    width = query_prop(PLAYER_I_SCREEN_WIDTH);
    if (!width)
	width = 80;
    set_screen_width(width);
}

/*
 * Function name: add_prop_player_i_screen_width
 * Description:   Autodetect when a player's screen width is modified
 *                and notify the game driver. Do not allow VBFC values.
 * Arguments:     val: The new property value
 */
public nomask int
add_prop_player_i_screen_width(mixed val)
{
    if (!intp(val))
	return 1;
    if (val)
	set_screen_width((int)val);
    else
	set_screen_width(80);
    return 0;
}

/*
 * Function name: enter_game
 * Description:   Enter the player into the game
 * Arguments:	  pl_name: Name of the player
 * Security:
 *           This function must only be called from "/secure/login"
 */
public nomask int
enter_game(string pl_name)
{
    string      fname;
    string	*souls;
    int 	il;
    int		lost_money;
    object 	ob;

    if (MASTER_OB(previous_object()) != "/secure/login" &&
	MASTER_OB(previous_object()) != LOGIN_NEW_PLAYER)
    {
	write("Bad login object: " + file_name(previous_object()) + "\n");
	return 0;
    }
#ifdef RELAY_HOST
    else
	real_host = previous_object()->query_relay();
#endif

    if (SECURITY->check_inherit("/std/player_sec", this_object()))
    {
	remove_object();
	return 0;
    }
    set_name(pl_name);

    fixup_screen();
    ::set_adj(({}));		/* No adjectives and no default */
    new_init();             	/* All variables to default condition */

    seteuid(0);
    if (!SECURITY->load_player())
    {
	return 0;
    }
    seteuid(query_wiz_level() ? pl_name : getuid(this_object()));
    set_adj(0);			/* Set the adjectives as loaded */

    add_name(query_race_name());
    if (query_wiz_level())
	add_name("wizard");

    /* Make some sanity things to guard against old and patched .o files
    */
    set_m_out(query_m_out());
    set_m_in(query_m_in());
    set_mm_out(query_mm_out());
    set_mm_in(query_mm_in());
    set_learn_pref(query_learn_pref(-1));

    /* People might have inconsistent acc_exp in guildstats, we must guard
       against it.
    */
    il = this_object()->query_exp();
    this_object()->add_exp(1, 1);
    if (this_object()->query_exp() < il)
    {
	il -= this_object()->query_exp();
	this_object()->add_exp(il);
    }
    else
	this_object()->add_exp(-1, 1);

    set_login_time();
    set_login_from();

    set_living_name(pl_name);
    cmd_sec_reset();
    
    acc_exp_to_stats();   /* Setup our current stats */

    if (query_temp_start_location() &&
	SECURITY->check_temp_start_loc(query_temp_start_location()) >= 0)
    {
	catch(this_object()->move_living(0, query_temp_start_location()));
	set_temp_start_location(0);
    }

    if (!environment(this_object()))
    {
	if (!query_default_start_location() ||
	    (!query_wiz_level() && SECURITY->
		check_def_start_loc(query_default_start_location()) < 0))
	    set_default_start_location((string) this_object()->
		query_def_start());
	catch(this_object()->move_living(0, query_default_start_location()));
    }

    /*
     * Let players start even if their start location is bad
     */
    if (!environment(this_object()) && !query_wiz_level())
	if (catch(this_object()->move_living(0, query_def_start())))
	{
	    /*
	     * If this start location is corrupt too, then destruct the player
	     */
	    write("PANIC, your starting locations are corrupt!!\n");
	    destruct();
    	}

    /* Set some default properties */
    this_object()->add_prop(PLAYER_I_MORE_LEN, 20);
    /* Restore the bits */
    this_object()->unpack_bits();

    if ((query_wiz_level()) && (environment(this_object())))
	write("Starting in:" + file_name(environment(this_object())) + "\n");

    init_saved_props();

    /* Non wizards should not have a lot of command souls
     */
    if (!query_wiz_level())
    {
	souls = this_object()->query_cmdsoul_list();
	if (sizeof(souls))
	{
	    for (il = 0; il < sizeof(souls); il++)
		remove_cmdsoul(souls[il]);
	}
    }

    /* Attach the quicktyper */
    if (!this_object()->query_cmdmodify())
    {
	this_object()->set_cmdmodify(this_object()); /* Built in QT */
    }

    if (!m_alias_list)
	m_alias_list = ([]);

    set_heart_beat(1);

    /* Start him up */
    if (query_autoshadow_list())
	load_auto_shadow(query_autoshadow_list());
    if (query_auto_load())
	load_auto_obj(query_auto_load());
    if (query_recover_list())
	if (load_recover_list(pl_name, query_recover_list()) == 0)
	    set_recover_list(({}));

    /* Start him up 
     */
    this_object()->start_player();
    this_object()->query_combat_object()->cb_configure();
    
    /* Non wizards should not have a lot of tool souls
     */
    if (!query_wiz_level())
    {
	souls = this_object()->query_tool_list();
	if (sizeof(souls))
	{
	    for (il = 0; il < sizeof(souls); il++)
		this_object()->remove_toolsoul(souls[il]);
	}
    }

    if ((lost_money = query_tot_value()) > 0)
    {
	tell_object(this_object(), "You find " + lost_money +
			" coppers of your lost money.\n");
    	MONEY_ADD(this_object(), query_tot_value());
    	call_out("save_me", 1, 0);
    }
    add_prop(PLAYER_I_LASTXP, query_exp());

    return 1;
}

static void
init_saved_props()
{
    int i, n;

    n = sizeof(SAVE_PROPS) < sizeof(saved_props) ? sizeof(SAVE_PROPS) :
	sizeof(saved_props);

    for (i = 0; i < n; i++)
	this_object()->add_prop(SAVE_PROPS[i], saved_props[i]);

    saved_props = ({ });
}

public nomask void
open_player()
{
    if ((previous_object() == find_object(SECURITY)) ||
	(MASTER_OB(previous_object()) == "/secure/login"))
	seteuid(0);
}

nomask public int
fix_saveprop_list()
{
    int i;

    /* Fix the saved_props list before save */
    saved_props = ({ });
    for (i = 0; i < sizeof(SAVE_PROPS); i++)
	saved_props += ({ this_object()->query_prop(SAVE_PROPS[i]) });
}

nomask public int
save_player(string pl_name)
{
    int ret;
   
    if (!pl_name)
	return 0;

    this_object()->pack_bits();
    seteuid(getuid(this_object()));
    save_object(PLAYER_FILE(pl_name));
    seteuid(getuid(this_object()));

    /* Discard the props again */
    saved_props = ({ });
    return 1;
}

nomask public int
load_player(string pl_name)
{
    int ret;
   
    if (!pl_name)
	return 0;

    seteuid(getuid(this_object()));
    ret = restore_object(PLAYER_FILE(pl_name));
    seteuid(0);
    return ret;
}

/*
 * Function name: load_auto_obj
 * Description:   Loads all autoloaded objects
 */
nomask static void
load_auto_obj(string *load_arr)
{
    string          file,
                    argument,
                    *parts;
    object          ob;
    int		    il;

    if (!load_arr || !sizeof(load_arr))
	return;

    for (il = 0; il < sizeof(load_arr); il++)
    {
	if (!stringp(load_arr[il]))
	{
	    write("Aotoload array element " + il + " corrupt.\n");
	    continue;
	}

	if (sscanf(load_arr[il], "%s:%s", file, argument) != 2)
	{
	    file = load_arr[il]; argument = 0;
	}
	if (!stringp(file))
	{
	    write("Auto load string corrupt: " + load_arr[il] + "\n");
	    continue;
	}
	if (LOAD_ERR(file))
	    continue;
	ob = find_object(file);
	if (!ob)
	    continue;
	ob = clone_object(file);
	if (argument)
	    ob->init_arg(argument);
	ob->move(this_object());
    }
}

/*
 * Function name: load_recover_list
 * Description:   Loads all recoverable objects.
 * Arguments:	  pl_name - the player name,
 *		  recover_arr - The array of objects to recover
 */
nomask static int
load_recover_list(string pl_name, string *recover_arr)
{
    string          file,
                    argument,
                    *parts;
    object          ob;
    int		    il;

    if (!recover_arr || !sizeof(recover_arr))
	return 0;

    /* Do not restore after 2 hours */
    if (time() - file_time(PLAYER_FILE(pl_name) + ".o") > 3600 * 2)
	return 0;

    for (il = 0; il < sizeof(recover_arr); il++)
    {
	if (sscanf(recover_arr[il], "%s:%s", file, argument) != 2)
	{
	    file = recover_arr[il]; argument = 0;
	}
	if (!stringp(file))
	{
	    write("Recover string corrupt: " + recover_arr[il] + "\n");
	    continue;
	}
	if (LOAD_ERR(file))
	    continue;
	ob = find_object(file);
	if (!ob)
	    continue;
	ob = clone_object(file);
        write("You recover your " + ob->short() + ".\n");
	if (argument)
	    ob->init_recover(argument);
	ob->move(this_object());
    }
}

/*
 * Function name: load_auto_shadow
 * Description:   Startup all the shadows that should shadow this player
 */
nomask static void
load_auto_shadow(string *load_arr)
{
    string          file,
                    argument,
                    *parts;
    object          ob;
    int		    il;

    if (!load_arr || !sizeof(load_arr))
	return;

    for (il = 0; il < sizeof(load_arr); il++)
    {
	if (sscanf(load_arr[il], "%s:%s", file, argument) != 2)
	{
	    write("Shadow load string corrupt: " + load_arr[il] + "\n");
	    continue;
	}
	if (LOAD_ERR(file))
	    continue;
	ob = find_object(file);
	if (!ob)
	    continue;
	ob = clone_object(file);
	if (argument)
	    ob->autoload_shadow(argument);
	else
	    ob->autoload_shadow(0);
    }
}

/*
 * Function name: new_save
 * Description:   Save this player as 'name'
 *                This is only called when new players
 *                enters the game. It makes it possible to initialize variables
 *                using the standard set_ calls.
 * Arguments:	  pl_name: Name of the player
 * Security:
 *           This function must only be called from "/secure/login" or
 *           from the object creating new players. Which inherits
 */
public nomask int
new_save(string pl_name, string pwd, string pfile)
{
    string          fname;

    if (!CALL_BY("/secure/login") &&
	!CALL_BY(LOGIN_NEW_PLAYER))
	return 0;

    write("Creating new player: " + pl_name + "\n");
    seteuid(getuid(this_object()));
    set_name(pl_name);
    set_password(pwd);
    set_player_file(pfile);
    new_init();			/* Initialize all variables on startup */

    save_object(PLAYER_FILE(pl_name));
    return 1;
}


/*
 * Function:     new_init
 * Description:  Initialises all variables to default conditions.
 */
static nomask void
new_init()
{
    int i, *ostat;

    ostat = query_orig_stat();
    for (i = 0; i < SS_NO_EXP_STATS; i++)
	set_base_stat(i, ostat[i]);

    stats_to_acc_exp();

    set_learn_pref(query_orig_learn());

    set_title(query_new_title());
#ifndef NO_ALIGN_TITLE
    set_al_title(query_new_al_title());
#endif
}

public nomask void
create_living()
{
    1;
}				/* We do not export any create call */

public nomask void
reset_living()
{
    1;
}				/* We do not export any reset call */

/*
 * Function name:	command
 * Description:		Makes the player execute a command, If the player is a
 *			wizard then the object must have the same effective
 *			userid as the wizard being forced.
 * Arguments:		cmd: String containing the command
 * Returns:		eval_cost or '0' if unsuccessful
 */
public nomask int
command(string cmd)
{
    object pobj;
    
    pobj = previous_object();
    if (query_wiz_level() && pobj && this_object())
        if (!SECURITY->wiz_force_check(geteuid(pobj), geteuid(this_object())))
	    return 0;

    return ::command(cmd);
}

nomask public int
follow_leader(string cmd, object leader)
{
    if (query_wiz_level())
    { /* Check for move commands */
	if (member_array(cmd, environment(this_object())->query_exit()) < 0)
	{
	    call_out("move_to_leader", 1);
	    return 0;
	}
    }
    return ::follow_leader(cmd, leader);
}

public nomask void
move_to_leader()
{
    if (query_leader())
	command("goto " + query_leader()->query_real_name());
}

public string
query_alias(string alias)
{
    mapping m;

    m = query_aliases();
    return(m[alias]);
}

public int
id(string str)
{
    if (notmet_me(previous_object()))
	if (str == query_real_name())
	    return 0;

    return ::id(str);
}
